"""
@file
@brief Missing information about licenses
"""

missing_module_licenses = {
    "xgboost": "Apache-2",
    "actuariat_python": "MIT",
    "code_beatrix": "MIT",
    "ensae_teaching_cs": "MIT",
    "pymyinstall": "MIT",
    "pyensae": "MIT",
    "pyquickhelper": "MIT",
    "pymmails": "MIT",
    "pyrsslocal": "MIT",
    "pysqllike": "MIT",
    "spyre": "MIT",
    "dataspyre": "MIT",
    "ete": "GNU v3",
    "ete3": "GNU v3",
    "datrie": "LGPL v2.1",
    "glueviz": "BSD",
    "h5py": "BSD",
    "statsmodels": "BSD",
    "toolz": "BSD",
    "trackpy": "BSD - 3 clauses",
    "jupyter_core": "BSD",
    "jupyter_client": "BSD",
    "metakernel": "BSD",
    "scilab_kernel": "BSD",
    "pyexecjs": "MIT",
    "selenium": "Apache Software License",
    "alabaster": "BSD",
    "cloud_sptheme": "BSD",
    "colorama": "BSD",
    "guzzle_sphinx_theme": "BSD~MIT",
    "itcase_sphinx_theme": "no license",
    "pypiserver": "BSD",
    "solar_theme": "BSD~MIT",
    "sphinx_bootstrap_theme": "MIT",
    "sphinx_py3doc_enhanced_theme": "BSD",
    "sphinx_readable_theme": "MIT",
    "sphinxjp.themes.basicstrap": "MIT",
    "sphinxjp.themes.sphinxjp": "MIT",
    "wild_sphinx_theme": "BSD",
    "SQLAlchemy": "MIT",
    "graphviz": "MIT",
    "anyjson": "BSD",
    "brewer2mpl ": "MIT",
    "cached_property": "BSD",
    "celery": "BSD",
    "colorspacious": "MIT",
    "cgal_bindings": "Boost Software License 1.0",
    "Cython": "Apache Software License",
    "datashape": "BSD",
    "cubehelix": "~BSD - 2 clauses",
    "dbfread": "MIT",
    "dynd": "BSD",
    "fabric": "BSD",
    "feedparser": "~BSD - 2 clauses",
    "gmpy2": "LGPLv3+",
    "itsdangerous": "BSD",
    "kombu": "BSD",
    "lifelines": "MIT",
    "line_profiler": "BSD",
    "lxml": "BSD",
    "Mako": "MIT",
    "marisa_trie": "MIT",
    "mock": "BSD",
    "msgpack": "Apache Software License",
    "multimethods": "MIT",
    "multipledispatch": "BSD",
    "nodeenv": "BSD",
    "oauthlib": "BSD",
    "patsy": "BSD",
    "pbr": "Apache Software License",
    "pims": "~BSD",
    "pipdeptree": "MIT",
    "PyAudio": "MIT",
    "pycrypto": "Public Domain + patent for some algorithm",
    "planar": "BSD",
    "pygal_maps_world": "LGPLv3+",
    "pymesos": "BSD",
    "pystache": "MIT",
    "pytz": "MIT",
    "requests_oauthlib": "ISC",
    "rope_py3k": "GPL=OpenBSD",
    "smart_open": "Public Domain",
    "textblob": "MIT",
    "tzlocal": "CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
    "viscm": "MIT",
    "zs": "BSD",
    "neural-python": "MIT",
    "dask": "BSD",
    "NLopt": "LGPL",
    "ipython_genutils": "BSD",
    "memory_profiler": "BSD",
    "snakeviz": "BSD",
    "ansi2html": "GPLv3+",
    "brewer2mpl": "MIT",
    "dev": "Public Domain",
    "django-model-utils": "BSD",
    "django-userena": "BSD",
    "django-uuidfield": "~BSD",
    "easy-thumbnails": "BSD",
    "gunicorn": "MIT",
    "lockfile": "MIT",
    "gevent": "MIT",
    "lz4": "BSD",
    "opencv_python": "BSD",
    "ptyprocess": "ISC",
    "terminado": "BSD",
    "easy_thumbnails": "BSD",
    "llvmpy": "BSD",
    "pysterior": "MIT",
    "pymc3": "Apache-2",
    "tqdm": "MIT",
    "pandas-highcharts": "MIT",
    "holoviews": "BSD",
    "cyordereddict": "MIT",
    "future": "MIT",
    "python-gmaps": "BSD",
    "libpython": "Python",
    "pyowm": "MIT",
    "PyMySQL": "MIT",
    "osmapi": "GPLv3",
    "progressbar2": "BSD",
    "wordcloud": "MIT",
    "django-contrib-comments": "BSD",
    "filebrowser_safe": "~MIT",
    "grappelli_safe": "~MIT",
    "mezzanine": "BSD",
    "affine": "BSD",
    "blocks": "GPL",
    "blz": "BSD",
    "chest": "BSD",
    "configobj": "BSD",
    "distributed": "BSD",
    "gatspy": "BSD",
    "guidata": "CeCILL v2",
    "guiqwt": "CECILL",
    "heapdict": "BSD",
    "httpretty": "MIT",
    "jaraco.structures": "MIT",
    "jdcal": "BSD",
    "jmespath": "MIT",
    "keyring": "MIT, Python",
    "locket": "BSD",
    "onedrive-sdk-python": "MIT",
    "pandocfilters": "BSD",
    "passlib": "BSD",
    "picklable-itertools": "MIT",
    "plac": "BSD",
    "pycryptodomex": "BSD",
    "PyOpenGL_accelerate": "BSD",
    "pyOpenSSL": "Apache 2.0",
    "pyRFXtrx": "LGPLv3+",
    "pysmi": "BSD",
    "pythonnet": "MIT",
    "pythonqwt": "MIT",
    "queuelib": "BSD",
    "semantic_version": "BSD",
    "slicerator": "BSD",
    "sqlite_bro": "MIT",
    "supersmoother": "BSD 3-clause",
    "tblib": "BSD",
    "unidecode": "GPLv2+",
    "vincenty": "none",
    "user-agent": "MIT",
    "wget": "none",
    "xxhash": "BSD",
    "zope.exceptions": "Zope Public License",
    "w3lib": "BSD",
    "astropy": "BSD",
    "backports_abc": "Python Software Foundation License",
    "boto3": "Apache Software License ",
    "botocore": "Apache Software License ",
    "scoop": "GNU Library or Lesser General Public License (LGPL)",
    "hikvision": "MIT",
    "PyMata": "GPLv3+",
    "python-mpd2": "LGPL",
    "python-nest": "Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License",
    "temperusb": "GNU GENERAL PUBLIC LICENSE",
    "sphinx-corlab-theme": "LGPLv3+",
    "sphinx-docs-theme": "MIT",
    "sphinxjp.themes.gopher": "MIT",
    "sphinxjp.themes.htmlslide": "MIT",
    "zerovm-sphinx-theme": "Apache License 2.0",
    "xarray": "Apache Software License",
    "typing": "PSF",
    "sklearn_pandas": "~MIT",
    "skll": "BSD",
    "abcpmc": "GPLv3+",
    "ad3": "GPLv3",
    "amqp": "LGPL",
    "ansiconv": "MIT",
    "apscheduler": "MIT",
    "autopy3": "MIT",
    "azure-batch-apps": "MIT",
    "azureml": "MIT",
    "bigfloat": "LGPLv3",
    "billiard": "BSD",
    "biopython": "~MIT",
    "bleach": "Apache",
    "blist": "BSD",
    "blosc": "MIT",
    "bqplot": "Apache",
    "btrees": "Zope Public License",
    "bz2file": "Apache",
    "Cartopy": "LGPLv3+",
    "chalmers": "MIT",
    "CherryPy": "BSD",
    "cobble": "BSD",
    "comtypes": "MIT",
    "contextlib2": "Python",
    "cssselect": "BSD",
    "cubes": "MIT",
    "cuda4py": "BSD",
    "CVXcanon": "?",
    "cvxpy": "GNU",
    "cymem": "MIT",
    "datashader": "BSD",
    "db.py": "BSD",
    "deap": "LGPL",
    "Django": "BSD",
    "django-audiotracks": "MIT",
    "django-celery": "BSD",
    "django-configurations": "BSD",
    "django-environ": "MIT",
    "django-guardian": "BSD",
    "django-storages": "BSD",
    "dlib": "boost",
    "dnspython": "~MIT",
    "docrepr": "Modified BSD",
    "entrypoints": "MIT",
    "envoy": "~MIT",
    "et_xmlfile": "MIT",
    "Flask-Login": "MIT",
    "Flask-SQLAlchemy": "BSD",
    "GDAL": "MIT",
    "gensim": "LGPLv2+",
    "geoplotlib": "MIT",
    "geopy": "MIT",
    "google-api-python-client": "Apache",
    "googlemaps": "Apache",
    "grab": "MIT",
    "greenlet": "MIT",
    "grequests": "BSD",
    "grin": "BSD",
    "HDDM": "BSD",
    "heatmap": "MIT",
    "Hebel": "GPLv2",
    "hmmlearn": "MIT + Copyright",
    "html2text": "GPL",
    "html5lib": "MIT",
    "httpie": "BSD",
    "imageio": "BSD",
    "imbox": "MIT",
    "invoke": "BSD",
    "ipyparallel": "BSD",
    "jedi": "MIT",
    "jieba": "MIT",
    "julia": "MIT",
    "jupytalk": "MIT",
    "Keras": "MIT",
    "Kivy": "MIT",
    "kivy-garden": "MIT",
    "luigi": "Apache 2.0",
    "ldap3": "LGPLv3",
    "lazy-object-proxy": "BSD",
    "AnyQt": "GPLv3",
    "Automat": "MIT",
    "azure-datalake-store": "MIT",
    "azure-keyvault": "MIT",
    "azure-mgmt-containerregistry": "MIT",
    "azure-mgmt-datalake-analytics": "MIT",
    "azure-mgmt-datalake-nspkg": "MIT",
    "azure-mgmt-datalake-store": "MIT",
    "azure-mgmt-devtestlabs": "MIT",
    "azure-mgmt-dns": "MIT",
    "azure-mgmt-documentdb": "MIT",
    "azure-mgmt-iothub": "MIT",
    "azure-mgmt-keyvault": "MIT",
    "azure-mgmt-monitor": "MIT",
    "azure-mgmt-rdbms": "MIT",
    "azure-mgmt-sql": "MIT",
    "azure-mgmt-trafficmanager": "MIT",
    "azure-servicefabric": "MIT",
    "backports.functools_lru_cache": "MIT",
    "bcrypt": "Apache License 2.0",
    "blockdiag": "Apache License 2.0",
    "Bottlechest": "Simplified BSD",
    "CacheControl": "Apache Software License",
    "category_encoders": "BSD",
    "Cheroot": "BSD",
    "colorcet": "Creative Commons Attribution 4.0 International Public License (CC-BY)",
    "colorlover": "MIT",
    "defusedxml": "Python Software Foundation License Version 2",
    "docx2txt": "MIT",
    "dominate": "GNU Lesser General Public License (LGPL)",
    "drawtree": "Apache License 2.0",
    "edward": "MIT",
    "elasticsearch": "Apache License 2.0",
    "ensae_projects": "MIT",
    "Fabric3": "BSD",
    "fastparquet": "Apache License 2.0",
    "fasttext": "BSD 3-Clause License",
    "fbprophet": "BSD",
    "filelock": "Unlicensed",
    "ftfy": "MIT",
    "funcparserlib": "MIT",
    "gitdb": "BSD",
    "GitPython": "BSD",
    "glue-core": "BSD",
    "glue-vispy-viewers": "As Is",
    "gvar": "GPLv3+",
    "h2": "MIT",
    "holopy": "GNU GENERAL PUBLIC LICENSE",
    "hpack": "MIT",
    "hyper": "MIT",
    "hyperframe": "MIT",
    "hyperlink": "MIT",
    "hyperspy": "GNU General Public License v3 (GPLv3)",
    "images2gif": "Unlicensed",
    "ipaddress": "Python Software Foundation License",
    "JPype1": "Apache Software License",
    "jsonpickle": "BSD",
    "jupyter_contrib_nbextensions ": "BSD",
    "jupyter_sphinx": "BSD",
    "jyquickhelper": "MIT",
    "kabuki": "As Is",
    "libLAS": "BSD",
    "lmfit": "BSD",
    "lsqfit": "GPLv3+",
    "lru-dict": "MIT",
    "mammoth": "BSD",
    "mbstrdecoder": "MIT",
    "mdn-sphinx-theme": "Mozilla Public License 2.0 (MPL 2.0)",
    "missingno": "As Is",
    "mkl-service": "Anaconda End User License Agreement",
    "mlstatpy": "MIT",
    "monotonic": "Apache",
    "monty": "MIT",
    "msgpack-numpy": "BSD",
    "nbdime": "BSD",
    "nose-parameterized": "BSD",
    "octave_kernel": "BSD",
    "odfpy": "GNU General Public License v.2 + Apache License v.2",
    "Orange3": "GPLv3+",
    "Orange3-Associate": "GPLv3+",
    "Orange3-ImageAnalytics": "GPLv3+",
    "Orange3-Network": "GPLv3+",
    "Orange3-Text": "GPLv3+",
    "palettable": "MIT",
    "path.py": "MIT",
    "pathvalidate": "MIT",
    "pdfminer3k": "MIT",
    "pdfrw": "MIT",
    "PIMS": "As Is",
    "Pint": "BSD",
    "pybars3": "GNU Library or Lesser General Public License (LGPL)",
    "pyclustering": "GNU General Public License v3 (GPLv3)",
    "pyemd": "MIT",
    "pyflux": "As Is",
    "pygal_maps_ch": "GNU LGPL v3+",
    "pygal_maps_fr": "GNU LGPL v3+",
    "pygal_sphinx_directives": "GNU LGPL v3+",
    "pyinstrument": "BSD",
    "pymatgen": "MIT",
    "pyopencl": "MIT",
    "PyOpenGL": "BSD",
    "pyPdf": "BSD",
    "PyPDF2": "BSD",
    "pypmc": "GPLv2",
    "pypng": "MIT",
    "pystan": "GNU General Public License v3 (GPLv3)",
    "PySide": "GNU Library or Lesser General Public License (LGPL)",
    "pytablereader": "MIT",
    "python-mimeparse": "MIT",
    "python3-linkedin": "MIT",
    "python-Levenshtein": "GNU General Public License v2 or later (GPLv2+)",
    "pythreejs": "BSD",
    "PyX": "GNU General Public License (GPL)",
    "qutip": "BSD",
    "recommonmark": "MIT",
    "regex": "Python Software Foundation License",
    "Rtree": "GNU Library or Lesser General Public License (LGPL)",
    "sacred": "MIT",
    "scikit-fusion": "GPLv3",
    "service_identity": "MIT",
    "setproctitle": "BSD",
    "setuptools-git": "BSD",
    "sdepy": "BSD",
    "simhash": "MIT",
    "SimpleSQLite": "MIT",
    "SIP": "GPL v2 or GPL v3 or BSD",
    "smmap": "BSD",
    "sounddevice": "MIT",
    "spglib": "BSD",
    "sphinx_theme_pd": "MIT",
    "sphinxcontrib-blockdiag": "BSD",
    "splinter": "As Is",
    "TA-Lib": "BSD",
    "teachpyx": "MIT",
    "tempora": "MIT",
    "tensorflow": "Apache Software License",
    "termcolor": "MIT",
    "testpath": "MIT",
    "tkinterquickhelper": "MIT",
    "toml": "MIT",
    "toyplot": "BSD",
    "traits": "BSD",
    "triangle": "GNU LGPL",
    "typepy": "MIT",
    "update_checker": "Simplified BSD License",
    "validate_email": "LGPL",
    "webcolors": "BSD",
    "wikipedia": "MIT",
    "win_unicode_console": "MIT",
    "xlwings": "BSD",
}
