# -*- coding: utf-8 -*-
"""
@file
@brief Implements command line ``python -m pymyinstall <args>``.
"""


if __name__ == "__main__":
    from pymyinstall.cli.pymy_install import main
    main()  # pragma: no cover
